@@ -9,6 +9,7 @@ import (
 	"time"
 
 	"github.com/hashicorp/cli"
+	"github.com/hashicorp/terraform/internal/backend"
 	"github.com/hashicorp/terraform/internal/command/arguments"
 	"github.com/hashicorp/terraform/internal/command/clistate"
 	"github.com/hashicorp/terraform/internal/command/views"
@@ -44,6 +45,11 @@ func (c *WorkspaceDeleteCommand) Run(args []string) int {
 		c.Ui.Error("Expected a single argument: NAME.\n")
 		return cli.RunResultHelp
 	}
+	if args[0] == "" {
+		// Disallowing empty string identifiers more explicitly, versus "Workspace "" doesn't exist."
+		c.Ui.Error(fmt.Sprintf("Expected a workspace name as an argument, instead got an empty string: %q\n", args[0]))
+		return cli.RunResultHelp
+	}
 
 	configPath, err := ModulePath(args[1:])
 	if err != nil {
@@ -79,6 +85,7 @@ func (c *WorkspaceDeleteCommand) Run(args []string) int {
 		return 1
 	}
 
+	// Is the user attempting to delete a workspace that doesn't exist?
 	workspace := args[0]
 	exists := false
 	for _, ws := range workspaces {
@@ -93,6 +100,7 @@ func (c *WorkspaceDeleteCommand) Run(args []string) int {
 		return 1
 	}
 
+	// Is the user attempting to delete the currently selected workspace?
 	currentWorkspace, err := c.Workspace()
 	if err != nil {
 		c.Ui.Error(fmt.Sprintf("Error selecting workspace: %s", err))
@@ -103,7 +111,13 @@ func (c *WorkspaceDeleteCommand) Run(args []string) int {
 		return 1
 	}
 
-	// we need the actual state to see if it's empty
+	// Is the user attempting to delete the default workspace?
+	if workspace == backend.DefaultStateName {
+		c.Ui.Error("Cannot delete the default workspace")
+		return 1
+	}
+
+	// Check if the workspace's state is empty or not
 	stateMgr, err := b.StateMgr(workspace)
 	if err != nil {
 		c.Ui.Error(err.Error())
