@@ -39,6 +39,10 @@ type Resource struct {
 	// For all other resource modes, this field is nil.
 	Managed *ManagedResource
 
+	// List is populated only for Mode = addrs.ListResourceMode,
+	// containing the additional fields that apply to list resources.
+	List *ListResource
+
 	// Container links a scoped resource back up to the resources that contains
 	// it. This field is referenced during static analysis to check whether any
 	// references are also made from within the same container.
@@ -52,8 +56,9 @@ type Resource struct {
 
 // ManagedResource represents a "resource" block in a module or file.
 type ManagedResource struct {
-	Connection   *Connection
-	Provisioners []*Provisioner
+	Connection     *Connection
+	Provisioners   []*Provisioner
+	ActionTriggers []*ActionTrigger
 
 	CreateBeforeDestroy bool
 	PreventDestroy      bool
@@ -64,6 +69,17 @@ type ManagedResource struct {
 	PreventDestroySet      bool
 }
 
+type ListResource struct {
+	// By default, the results of a list resource only include the identities of
+	// the discovered resources. If the user specifies "include_resources = true",
+	// then the provider should include the resource data in the result.
+	IncludeResource hcl.Expression
+
+	// Limit is an optional expression that can be used to limit the
+	// number of results returned by the list resource.
+	Limit hcl.Expression
+}
+
 func (r *Resource) moduleUniqueKey() string {
 	return r.Addr().String()
 }
@@ -104,7 +120,7 @@ func (r *Resource) HasCustomConditions() bool {
 	return len(r.Postconditions) != 0 || len(r.Preconditions) != 0
 }
 
-func decodeResourceBlock(block *hcl.Block, override bool) (*Resource, hcl.Diagnostics) {
+func decodeResourceBlock(block *hcl.Block, override bool, allowExperiments bool) (*Resource, hcl.Diagnostics) {
 	var diags hcl.Diagnostics
 	r := &Resource{
 		Mode:      addrs.ManagedResourceMode,
@@ -281,6 +297,17 @@ func decodeResourceBlock(block *hcl.Block, override bool) (*Resource, hcl.Diagno
 					case "postcondition":
 						r.Postconditions = append(r.Postconditions, cr)
 					}
+
+				// decoded, but not yet used!
+				case "action_trigger":
+					if allowExperiments {
+						at, atDiags := decodeActionTriggerBlock(block)
+						diags = append(diags, atDiags...)
+						if at != nil {
+							r.Managed.ActionTriggers = append(r.Managed.ActionTriggers, at)
+						}
+					}
+
 				default:
 					// The cases above should be exhaustive for all block types
 					// defined in the lifecycle schema, so this shouldn't happen.
@@ -963,5 +990,6 @@ var resourceLifecycleBlockSchema = &hcl.BodySchema{
 	Blocks: []hcl.BlockHeaderSchema{
 		{Type: "precondition"},
 		{Type: "postcondition"},
+		{Type: "action_trigger"},
 	},
 }
