@@ -14,20 +14,27 @@ import (
 	"github.com/zclconf/go-cty/cty"
 	"github.com/zclconf/go-cty/cty/convert"
 
-	"maps"
-
 	"github.com/hashicorp/terraform/internal/addrs"
+	"github.com/hashicorp/terraform/internal/backend/backendrun"
 	"github.com/hashicorp/terraform/internal/command/views"
 	"github.com/hashicorp/terraform/internal/configs"
 	"github.com/hashicorp/terraform/internal/didyoumean"
 	"github.com/hashicorp/terraform/internal/lang"
 	"github.com/hashicorp/terraform/internal/lang/langrefs"
 	"github.com/hashicorp/terraform/internal/moduletest"
-	hcltest "github.com/hashicorp/terraform/internal/moduletest/hcl"
+	"github.com/hashicorp/terraform/internal/providers"
+	"github.com/hashicorp/terraform/internal/states"
 	"github.com/hashicorp/terraform/internal/terraform"
 	"github.com/hashicorp/terraform/internal/tfdiags"
 )
 
+// TestFileState is a helper struct that just maps a run block to the state that
+// was produced by the execution of that run block.
+type TestFileState struct {
+	Run   *moduletest.Run
+	State *states.State
+}
+
 // EvalContext is a container for context relating to the evaluation of a
 // particular .tftest.hcl file.
 // This context is used to track the various values that are available to the
@@ -35,25 +42,22 @@ import (
 // within the suite.
 // The struct provides concurrency-safe access to the various maps it contains.
 type EvalContext struct {
-	VariableCaches *hcltest.VariableCaches
+	// unparsedVariables and parsedVariables are the values for the variables
+	// required by this test file. The parsedVariables will be populated as the
+	// test graph is executed, while the unparsedVariables will be lazily
+	// evaluated by each run block that needs them.
+	unparsedVariables map[string]backendrun.UnparsedVariableValue
+	parsedVariables   terraform.InputValues
+	variableStatus    map[string]moduletest.Status
+	variablesLock     sync.Mutex
 
-	// runOutputs is a mapping from run addresses to cty object values
-	// representing the collected output values from the module under test.
-	//
-	// This is used to allow run blocks to refer back to the output values of
-	// previous run blocks. It is passed into the Evaluate functions that
-	// validate the test assertions, and used when calculating values for
-	// variables within run blocks.
-	runOutputs  map[addrs.Run]cty.Value
+	// runBlocks caches all the known run blocks that this EvalContext manages.
+	runBlocks   map[string]*moduletest.Run
 	outputsLock sync.Mutex
 
-	// configProviders is a cache of config keys mapped to all the providers
-	// referenced by the given config.
-	//
-	// The config keys are globally unique across an entire test suite, so we
-	// store this at the suite runner level to get maximum efficiency.
-	configProviders map[string]map[string]bool
-	providersLock   sync.Mutex
+	providers      map[addrs.RootProviderConfig]providers.Interface
+	providerStatus map[addrs.RootProviderConfig]moduletest.Status
+	providersLock  sync.Mutex
 
 	// FileStates is a mapping of module keys to it's last applied state
 	// file.
@@ -72,38 +76,50 @@ type EvalContext struct {
 	stopContext   context.Context
 	stopFunc      context.CancelFunc
 
+	config   *configs.Config
 	renderer views.Test
 	verbose  bool
+
+	evalSem terraform.Semaphore
 }
 
 type EvalContextOpts struct {
-	Verbose   bool
-	Render    views.Test
-	CancelCtx context.Context
-	StopCtx   context.Context
+	Verbose           bool
+	Render            views.Test
+	CancelCtx         context.Context
+	StopCtx           context.Context
+	UnparsedVariables map[string]backendrun.UnparsedVariableValue
+	Config            *configs.Config
+	Concurrency       int
 }
 
 // NewEvalContext constructs a new graph evaluation context for use in
 // evaluating the runs within a test suite.
 // The context is initialized with the provided cancel and stop contexts, and
 // these contexts can be used from external commands to signal the termination of the test suite.
-func NewEvalContext(opts *EvalContextOpts) *EvalContext {
+func NewEvalContext(opts EvalContextOpts) *EvalContext {
 	cancelCtx, cancel := context.WithCancel(opts.CancelCtx)
 	stopCtx, stop := context.WithCancel(opts.StopCtx)
 	return &EvalContext{
-		runOutputs:      make(map[addrs.Run]cty.Value),
-		outputsLock:     sync.Mutex{},
-		configProviders: make(map[string]map[string]bool),
-		providersLock:   sync.Mutex{},
-		FileStates:      make(map[string]*TestFileState),
-		stateLock:       sync.Mutex{},
-		VariableCaches:  hcltest.NewVariableCaches(),
-		cancelContext:   cancelCtx,
-		cancelFunc:      cancel,
-		stopContext:     stopCtx,
-		stopFunc:        stop,
-		verbose:         opts.Verbose,
-		renderer:        opts.Render,
+		unparsedVariables: opts.UnparsedVariables,
+		parsedVariables:   make(terraform.InputValues),
+		variableStatus:    make(map[string]moduletest.Status),
+		variablesLock:     sync.Mutex{},
+		runBlocks:         make(map[string]*moduletest.Run),
+		outputsLock:       sync.Mutex{},
+		providers:         make(map[addrs.RootProviderConfig]providers.Interface),
+		providerStatus:    make(map[addrs.RootProviderConfig]moduletest.Status),
+		providersLock:     sync.Mutex{},
+		FileStates:        make(map[string]*TestFileState),
+		stateLock:         sync.Mutex{},
+		cancelContext:     cancelCtx,
+		cancelFunc:        cancel,
+		stopContext:       stopCtx,
+		stopFunc:          stop,
+		verbose:           opts.Verbose,
+		renderer:          opts.Render,
+		config:            opts.Config,
+		evalSem:           terraform.NewSemaphore(opts.Concurrency),
 	}
 }
 
@@ -139,6 +155,93 @@ func (ec *EvalContext) Verbose() bool {
 	return ec.verbose
 }
 
+func (ec *EvalContext) HclContext(references []*addrs.Reference) (*hcl.EvalContext, tfdiags.Diagnostics) {
+	var diags tfdiags.Diagnostics
+
+	runs := make(map[string]cty.Value)
+	vars := make(map[string]cty.Value)
+
+	for _, reference := range references {
+		switch subject := reference.Subject.(type) {
+		case addrs.Run:
+			run, ok := ec.GetOutput(subject.Name)
+			if !ok {
+				diags = diags.Append(&hcl.Diagnostic{
+					Severity: hcl.DiagError,
+					Summary:  "Reference to unknown run block",
+					Detail:   fmt.Sprintf("The run block %q does not exist within this test file.", subject.Name),
+					Subject:  reference.SourceRange.ToHCL().Ptr(),
+				})
+				continue
+			}
+			runs[subject.Name] = run
+
+			value, valueDiags := reference.Remaining.TraverseRel(run)
+			diags = diags.Append(valueDiags)
+			if valueDiags.HasErrors() {
+				continue
+			}
+
+			if !value.IsWhollyKnown() {
+				// This is not valid, we cannot allow users to pass unknown
+				// values into references within the test file. There's just
+				// going to be difficult and confusing errors later if this
+				// happens.
+				//
+				// When reporting this we assume that it's happened because
+				// the prior run was a plan-only run and that some of its
+				// output values were not known. If this arises for a
+				// run that performed a full apply then this is a bug in
+				// Terraform's modules runtime, because unknown output
+				// values should not be possible in that case.
+				diags = diags.Append(&hcl.Diagnostic{
+					Severity: hcl.DiagError,
+					Summary:  "Reference to unknown value",
+					Detail:   fmt.Sprintf("The value for %s is unknown. Run block %q is executing a \"plan\" operation, and the specified output value is only known after apply.", reference.DisplayString(), subject.Name),
+					Subject:  reference.SourceRange.ToHCL().Ptr(),
+				})
+				continue
+			}
+
+		case addrs.InputVariable:
+			if variable, ok := ec.GetVariable(subject.Name); ok {
+				vars[subject.Name] = variable.Value
+				continue
+			}
+
+			if variable, moreDiags := ec.EvaluateUnparsedVariableDeprecated(subject.Name, reference); variable != nil {
+				diags = diags.Append(moreDiags)
+				vars[subject.Name] = variable.Value
+				continue
+			}
+
+			diags = diags.Append(&hcl.Diagnostic{
+				Severity: hcl.DiagError,
+				Summary:  "Reference to unavailable variable",
+				Detail:   fmt.Sprintf("The input variable %q does not exist within this test file.", subject.Name),
+				Subject:  reference.SourceRange.ToHCL().Ptr(),
+			})
+			continue
+
+		default:
+			diags = diags.Append(&hcl.Diagnostic{
+				Severity: hcl.DiagError,
+				Summary:  "Invalid reference",
+				Detail:   "You can only reference run blocks and variables from within Terraform Test files.",
+				Subject:  reference.SourceRange.ToHCL().Ptr(),
+			})
+		}
+	}
+
+	return &hcl.EvalContext{
+		Variables: map[string]cty.Value{
+			"run": cty.ObjectVal(runs),
+			"var": cty.ObjectVal(vars),
+		},
+		Functions: lang.TestingFunctions(),
+	}, diags
+}
+
 // EvaluateRun processes the assertions inside the provided configs.TestRun against
 // the run results, returning a status, an object value representing the output
 // values from the module under test, and diagnostics describing any problems.
@@ -305,41 +408,137 @@ func (ec *EvalContext) EvaluateRun(run *moduletest.Run, resultScope *lang.Scope,
 	return status, cty.ObjectVal(outputVals), diags
 }
 
-func (ec *EvalContext) SetOutput(run *moduletest.Run, output cty.Value) {
-	ec.outputsLock.Lock()
-	defer ec.outputsLock.Unlock()
-	ec.runOutputs[run.Addr()] = output
-}
-
-func (ec *EvalContext) GetOutputs() map[addrs.Run]cty.Value {
-	ec.outputsLock.Lock()
-	defer ec.outputsLock.Unlock()
-	outputCopy := make(map[addrs.Run]cty.Value, len(ec.runOutputs))
-	maps.Copy(outputCopy, ec.runOutputs) // don't use clone here, so we can return a non-nil map
-	return outputCopy
-}
-
-func (ec *EvalContext) GetCache(run *moduletest.Run) *hcltest.VariableCache {
-	return ec.VariableCaches.GetCache(run.Name, run.ModuleConfig)
-}
-
-// ProviderExists returns true if the provider exists for the run inside the context.
-func (ec *EvalContext) ProviderExists(run *moduletest.Run, key string) bool {
-	ec.providersLock.Lock()
-	defer ec.providersLock.Unlock()
-	runProviders, ok := ec.configProviders[run.GetModuleConfigID()]
-	if !ok {
-		return false
+// EvaluateUnparsedVariable accepts a variable name and a variable definition
+// and checks if we have external unparsed variables that match the given
+// configuration. If no variable was provided, we'll return a nil
+// input value.
+func (ec *EvalContext) EvaluateUnparsedVariable(name string, config *configs.Variable) (*terraform.InputValue, tfdiags.Diagnostics) {
+	variable, exists := ec.unparsedVariables[name]
+	if !exists {
+		return nil, nil
 	}
 
-	found, ok := runProviders[key]
-	return ok && found
+	value, diags := variable.ParseVariableValue(config.ParsingMode)
+	if diags.HasErrors() {
+		value = &terraform.InputValue{
+			Value: cty.DynamicVal,
+		}
+	}
+
+	return value, diags
 }
 
-func (ec *EvalContext) SetProviders(run *moduletest.Run, providers map[string]bool) {
+// EvaluateUnparsedVariableDeprecated accepts a variable name without a variable
+// definition and attempts to parse it.
+//
+// This function represents deprecated functionality within the testing
+// framework. It is no longer valid to reference external variables without a
+// definition, but we do our best here and provide a warning that this will
+// become completely unsupported in the future.
+func (ec *EvalContext) EvaluateUnparsedVariableDeprecated(name string, ref *addrs.Reference) (*terraform.InputValue, tfdiags.Diagnostics) {
+	variable, exists := ec.unparsedVariables[name]
+	if !exists {
+		return nil, nil
+	}
+
+	var diags tfdiags.Diagnostics
+	diags = diags.Append(&hcl.Diagnostic{
+		Severity: hcl.DiagWarning,
+		Summary:  "Variable referenced without definition",
+		Detail:   fmt.Sprintf("Variable %q was referenced without providing a definition. Referencing undefined variables within Terraform Test files is deprecated, please add a `variable` block into the relevant test file to provide a definition for the variable. This will become required in future versions of Terraform.", name),
+		Subject:  ref.SourceRange.ToHCL().Ptr(),
+	})
+
+	// For backwards-compatibility reasons we do also have to support trying
+	// to parse the global variables without a configuration. We introduced the
+	// file-level variable definitions later, and users were already using
+	// global variables so we do need to keep supporting this use case.
+
+	// Otherwise, we have no configuration so we're going to try both parsing
+	// modes.
+
+	value, moreDiags := variable.ParseVariableValue(configs.VariableParseHCL)
+	diags = diags.Append(moreDiags)
+	if !moreDiags.HasErrors() {
+		// then good! we can just return these values directly.
+		return value, diags
+	}
+
+	// otherwise, we'll try the other one.
+
+	value, moreDiags = variable.ParseVariableValue(configs.VariableParseLiteral)
+	diags = diags.Append(moreDiags)
+	if moreDiags.HasErrors() {
+		// as usual make sure we still provide something for this value.
+		value = &terraform.InputValue{
+			Value: cty.DynamicVal,
+		}
+	}
+	return value, diags
+}
+
+func (ec *EvalContext) SetVariable(name string, val *terraform.InputValue) {
+	ec.variablesLock.Lock()
+	defer ec.variablesLock.Unlock()
+
+	ec.parsedVariables[name] = val
+}
+
+func (ec *EvalContext) GetVariable(name string) (*terraform.InputValue, bool) {
+	ec.variablesLock.Lock()
+	defer ec.variablesLock.Unlock()
+
+	variable, ok := ec.parsedVariables[name]
+	return variable, ok
+}
+
+func (ec *EvalContext) SetVariableStatus(address string, status moduletest.Status) {
+	ec.variablesLock.Lock()
+	defer ec.variablesLock.Unlock()
+	ec.variableStatus[address] = status
+}
+
+func (ec *EvalContext) AddRunBlock(run *moduletest.Run) {
+	ec.outputsLock.Lock()
+	defer ec.outputsLock.Unlock()
+	ec.runBlocks[run.Name] = run
+}
+
+func (ec *EvalContext) GetOutput(name string) (cty.Value, bool) {
+	ec.outputsLock.Lock()
+	defer ec.outputsLock.Unlock()
+	output, ok := ec.runBlocks[name]
+	if !ok {
+		return cty.NilVal, false
+	}
+	return output.Outputs, true
+}
+
+func (ec *EvalContext) ProviderForConfigAddr(addr addrs.LocalProviderConfig) addrs.Provider {
+	return ec.config.ProviderForConfigAddr(addr)
+}
+
+func (ec *EvalContext) LocalNameForProvider(addr addrs.RootProviderConfig) string {
+	return ec.config.Module.LocalNameForProvider(addr.Provider)
+}
+
+func (ec *EvalContext) GetProvider(addr addrs.RootProviderConfig) (providers.Interface, bool) {
 	ec.providersLock.Lock()
 	defer ec.providersLock.Unlock()
-	ec.configProviders[run.GetModuleConfigID()] = providers
+	provider, ok := ec.providers[addr]
+	return provider, ok
+}
+
+func (ec *EvalContext) SetProvider(addr addrs.RootProviderConfig, provider providers.Interface) {
+	ec.providersLock.Lock()
+	defer ec.providersLock.Unlock()
+	ec.providers[addr] = provider
+}
+
+func (ec *EvalContext) SetProviderStatus(addr addrs.RootProviderConfig, status moduletest.Status) {
+	ec.providersLock.Lock()
+	defer ec.providersLock.Unlock()
+	ec.providerStatus[addr] = status
 }
 
 func diagsForEphemeralResources(refs []*addrs.Reference) (diags tfdiags.Diagnostics) {
@@ -374,6 +573,80 @@ func (ec *EvalContext) GetFileState(key string) *TestFileState {
 	return ec.FileStates[key]
 }
 
+// ReferencesCompleted returns true if all the listed references were actually
+// executed successfully. This allows nodes in the graph to decide if they
+// should execute or not based on the status of their references.
+func (ec *EvalContext) ReferencesCompleted(refs []*addrs.Reference) bool {
+	for _, ref := range refs {
+		switch ref := ref.Subject.(type) {
+		case addrs.Run:
+			ec.outputsLock.Lock()
+			if run, ok := ec.runBlocks[ref.Name]; ok {
+				if run.Status != moduletest.Pass && run.Status != moduletest.Fail {
+					ec.outputsLock.Unlock()
+
+					// see also prior runs completed
+
+					return false
+				}
+			}
+			ec.outputsLock.Unlock()
+		case addrs.InputVariable:
+			ec.variablesLock.Lock()
+			if vStatus, ok := ec.variableStatus[ref.Name]; ok && (vStatus == moduletest.Skip || vStatus == moduletest.Error) {
+				ec.variablesLock.Unlock()
+				return false
+			}
+			ec.variablesLock.Unlock()
+		}
+	}
+	return true
+}
+
+// ProvidersCompleted ensures that all required providers were properly
+// initialised.
+func (ec *EvalContext) ProvidersCompleted(providers map[addrs.RootProviderConfig]providers.Interface) bool {
+	ec.providersLock.Lock()
+	defer ec.providersLock.Unlock()
+
+	for provider := range providers {
+		if status, ok := ec.providerStatus[provider]; ok {
+			if status == moduletest.Skip || status == moduletest.Error {
+				return false
+			}
+		}
+	}
+	return true
+}
+
+// PriorRunsCompleted checks a list of run blocks against our internal log of
+// completed run blocks and makes sure that any that do exist successfully
+// executed to completion.
+//
+// Note that run blocks that are not in the list indicate a bad reference,
+// which we ignore here. This is actually the problem of the caller to identify
+// and error.
+func (ec *EvalContext) PriorRunsCompleted(runs map[string]*moduletest.Run) bool {
+	ec.outputsLock.Lock()
+	defer ec.outputsLock.Unlock()
+
+	for name := range runs {
+		if run, ok := ec.runBlocks[name]; ok {
+			if run.Status != moduletest.Pass && run.Status != moduletest.Fail {
+
+				// pass and fail indicate the run block still executed the plan
+				// or apply operate and wrote outputs. fail means the
+				// post-execution checks failed, but we still had data to check.
+				// this is in contrast to pending, skip, or error which indicate
+				// that we never even wrote data for this run block.
+
+				return false
+			}
+		}
+	}
+	return true
+}
+
 // evaluationData augments an underlying lang.Data -- presumably resulting
 // from a terraform.Context.PlanAndEval or terraform.Context.ApplyAndEval call --
 // with results from prior runs that should therefore be available when
@@ -438,7 +711,7 @@ func (d *evaluationData) GetResource(addr addrs.Resource, rng tfdiags.SourceRang
 // GetRunBlock implements lang.Data.
 func (d *evaluationData) GetRunBlock(addr addrs.Run, rng tfdiags.SourceRange) (cty.Value, tfdiags.Diagnostics) {
 	var diags tfdiags.Diagnostics
-	ret, exists := d.ctx.GetOutputs()[addr]
+	ret, exists := d.ctx.GetOutput(addr.Name)
 	if !exists {
 		ret = cty.DynamicVal
 		diags = diags.Append(&hcl.Diagnostic{
@@ -484,15 +757,17 @@ func (d *evaluationData) StaticValidateReferences(refs []*addrs.Reference, self
 }
 
 func (d *evaluationData) staticValidateRunRef(ref *addrs.Reference) tfdiags.Diagnostics {
+	d.ctx.outputsLock.Lock()
+	defer d.ctx.outputsLock.Unlock()
+
 	var diags tfdiags.Diagnostics
 
 	addr := ref.Subject.(addrs.Run)
-	outputs := d.ctx.GetOutputs()
-	_, exists := outputs[addr]
-	if !exists {
+
+	if _, exists := d.ctx.runBlocks[addr.Name]; !exists {
 		var suggestions []string
-		for altAddr := range outputs {
-			suggestions = append(suggestions, altAddr.Name)
+		for altAddr := range d.ctx.runBlocks {
+			suggestions = append(suggestions, altAddr)
 		}
 		sort.Strings(suggestions)
 		suggestion := didyoumean.NameSuggestion(addr.Name, suggestions)
