@@ -5,14 +5,19 @@ package plugin
 
 import (
 	"bytes"
+	"context"
 	"fmt"
+	"io"
 	"testing"
 	"time"
 
 	"github.com/google/go-cmp/cmp"
 	"github.com/hashicorp/terraform/internal/addrs"
+	"github.com/hashicorp/terraform/internal/configs/configschema"
 	"github.com/hashicorp/terraform/internal/configs/hcl2shim"
+	"github.com/hashicorp/terraform/internal/plans"
 	"github.com/hashicorp/terraform/internal/providers"
+	"github.com/hashicorp/terraform/internal/schemarepo"
 	"github.com/hashicorp/terraform/internal/tfdiags"
 	"github.com/zclconf/go-cty/cty"
 	"go.uber.org/mock/gomock"
@@ -20,6 +25,7 @@ import (
 	"google.golang.org/grpc/status"
 	"google.golang.org/protobuf/types/known/timestamppb"
 
+	"github.com/hashicorp/terraform/internal/plugin/convert"
 	mockproto "github.com/hashicorp/terraform/internal/plugin/mock_proto"
 	proto "github.com/hashicorp/terraform/internal/tfplugin5"
 )
@@ -89,6 +95,18 @@ func providerProtoSchema() *proto.GetProviderSchema_Response {
 					},
 				},
 			},
+			"list": {
+				Version: 1,
+				Block: &proto.Schema_Block{
+					Attributes: []*proto.Schema_Attribute{
+						{
+							Name:     "resource_attr",
+							Type:     []byte(`"string"`),
+							Required: true,
+						},
+					},
+				},
+			},
 		},
 		DataSourceSchemas: map[string]*proto.Schema{
 			"data": &proto.Schema{
@@ -117,6 +135,20 @@ func providerProtoSchema() *proto.GetProviderSchema_Response {
 				},
 			},
 		},
+		ListResourceSchemas: map[string]*proto.Schema{
+			"list": &proto.Schema{
+				Version: 1,
+				Block: &proto.Schema_Block{
+					Attributes: []*proto.Schema_Attribute{
+						{
+							Name:     "filter_attr",
+							Type:     []byte(`"string"`),
+							Required: true,
+						},
+					},
+				},
+			},
+		},
 		ServerCapabilities: &proto.ServerCapabilities{
 			GetProviderSchemaOptional: true,
 		},
@@ -136,6 +168,16 @@ func providerResourceIdentitySchemas() *proto.GetResourceIdentitySchemas_Respons
 					},
 				},
 			},
+			"list": {
+				Version: 1,
+				IdentityAttributes: []*proto.ResourceIdentitySchema_IdentityAttribute{
+					{
+						Name:              "id_attr",
+						Type:              []byte(`"string"`),
+						RequiredForImport: true,
+					},
+				},
+			},
 		},
 	}
 }
@@ -398,6 +440,61 @@ func TestGRPCProvider_ValidateDataSourceConfig(t *testing.T) {
 	checkDiags(t, resp.Diagnostics)
 }
 
+func TestGRPCProvider_ValidateListResourceConfig(t *testing.T) {
+	client := mockProviderClient(t)
+	p := &GRPCProvider{
+		client: client,
+	}
+
+	client.EXPECT().ValidateListResourceConfig(
+		gomock.Any(),
+		gomock.Any(),
+	).Return(&proto.ValidateListResourceConfig_Response{}, nil)
+
+	cfg := hcl2shim.HCL2ValueFromConfigValue(map[string]interface{}{"config": map[string]interface{}{"filter_attr": "value"}})
+	resp := p.ValidateListResourceConfig(providers.ValidateListResourceConfigRequest{
+		TypeName: "list",
+		Config:   cfg,
+	})
+	checkDiags(t, resp.Diagnostics)
+}
+
+func TestGRPCProvider_ValidateListResourceConfig_OptionalCfg(t *testing.T) {
+	ctrl := gomock.NewController(t)
+	client := mockproto.NewMockProviderClient(ctrl)
+	sch := providerProtoSchema()
+	sch.ListResourceSchemas["list"].Block.Attributes[0].Optional = true
+	sch.ListResourceSchemas["list"].Block.Attributes[0].Required = false
+	// we always need a GetSchema method
+	client.EXPECT().GetSchema(
+		gomock.Any(),
+		gomock.Any(),
+		gomock.Any(),
+	).Return(sch, nil)
+
+	// GetResourceIdentitySchemas is called as part of GetSchema
+	client.EXPECT().GetResourceIdentitySchemas(
+		gomock.Any(),
+		gomock.Any(),
+		gomock.Any(),
+	).Return(providerResourceIdentitySchemas(), nil)
+
+	p := &GRPCProvider{
+		client: client,
+	}
+	client.EXPECT().ValidateListResourceConfig(
+		gomock.Any(),
+		gomock.Any(),
+	).Return(&proto.ValidateListResourceConfig_Response{}, nil)
+
+	cfg := hcl2shim.HCL2ValueFromConfigValue(map[string]interface{}{})
+	resp := p.ValidateListResourceConfig(providers.ValidateListResourceConfigRequest{
+		TypeName: "list",
+		Config:   cfg,
+	})
+	checkDiags(t, resp.Diagnostics)
+}
+
 func TestGRPCProvider_UpgradeResourceState(t *testing.T) {
 	client := mockProviderClient(t)
 	p := &GRPCProvider{
@@ -1298,3 +1395,399 @@ func TestGRPCProvider_closeEphemeralResource(t *testing.T) {
 
 	checkDiags(t, resp.Diagnostics)
 }
+
+func TestGRPCProvider_GetSchema_ListResourceTypes(t *testing.T) {
+	p := &GRPCProvider{
+		client: mockProviderClient(t),
+		ctx:    context.Background(),
+	}
+
+	resp := p.GetProviderSchema()
+	listResourceSchema := resp.ListResourceTypes
+	expected := map[string]providers.Schema{
+		"list": {
+			Version: 1,
+			Body: &configschema.Block{
+				Attributes: map[string]*configschema.Attribute{
+					"data": {
+						Type:     cty.DynamicPseudoType,
+						Computed: true,
+					},
+				},
+				BlockTypes: map[string]*configschema.NestedBlock{
+					"config": {
+						Block: configschema.Block{
+							Attributes: map[string]*configschema.Attribute{
+								"filter_attr": {
+									Type:     cty.String,
+									Required: true,
+								},
+							},
+						},
+						Nesting: configschema.NestingSingle,
+					},
+				},
+			},
+		},
+	}
+	checkDiags(t, resp.Diagnostics)
+
+	actualBody := convert.ConfigSchemaToProto(listResourceSchema["list"].Body).String()
+	expectedBody := convert.ConfigSchemaToProto(expected["list"].Body).String()
+	if actualBody != expectedBody {
+		t.Fatalf("expected %v, got %v", expectedBody, actualBody)
+	}
+}
+
+func TestGRPCProvider_Encode(t *testing.T) {
+	// TODO: This is the only test in this package that imports plans. If that
+	// ever leads to a circular import, we should consider moving this test to
+	// a different package or refactoring the test to not use plans.
+	p := &GRPCProvider{
+		client: mockProviderClient(t),
+		ctx:    context.Background(),
+		Addr:   addrs.ImpliedProviderForUnqualifiedType("testencode"),
+	}
+	resp := p.GetProviderSchema()
+
+	src := plans.NewChanges()
+	src.SyncWrapper().AppendResourceInstanceChange(&plans.ResourceInstanceChange{
+		Addr: addrs.AbsResourceInstance{
+			Module: addrs.RootModuleInstance,
+			Resource: addrs.ResourceInstance{
+				Resource: addrs.Resource{
+					Mode: addrs.ListResourceMode,
+					Type: "list",
+					Name: "test",
+				},
+				Key: addrs.NoKey,
+			},
+		},
+		ProviderAddr: addrs.AbsProviderConfig{
+			Provider: p.Addr,
+		},
+		Change: plans.Change{
+			Before: cty.NullVal(cty.Object(map[string]cty.Type{
+				"config": cty.Object(map[string]cty.Type{
+					"filter_attr": cty.String,
+				}),
+				"data": cty.List(cty.Object(map[string]cty.Type{
+					"state": cty.Object(map[string]cty.Type{
+						"resource_attr": cty.String,
+					}),
+					"identity": cty.Object(map[string]cty.Type{
+						"id_attr": cty.String,
+					}),
+				})),
+			})),
+			After: cty.ObjectVal(map[string]cty.Value{
+				"config": cty.ObjectVal(map[string]cty.Value{
+					"filter_attr": cty.StringVal("value"),
+				}),
+				"data": cty.ListVal([]cty.Value{
+					cty.ObjectVal(map[string]cty.Value{
+						"state": cty.ObjectVal(map[string]cty.Value{
+							"resource_attr": cty.StringVal("value"),
+						}),
+						"identity": cty.ObjectVal(map[string]cty.Value{
+							"id_attr": cty.StringVal("value"),
+						}),
+					}),
+				}),
+			}),
+		},
+	})
+	_, err := src.Encode(&schemarepo.Schemas{
+		Providers: map[addrs.Provider]providers.ProviderSchema{
+			p.Addr: {
+				ResourceTypes:     resp.ResourceTypes,
+				ListResourceTypes: resp.ListResourceTypes,
+			},
+		},
+	})
+	if err != nil {
+		t.Fatalf("unexpected error encoding changes: %s", err)
+	}
+}
+
+// Mock implementation of the ListResource stream client
+type mockListResourceStreamClient struct {
+	events  []*proto.ListResource_Event
+	current int
+	proto.Provider_ListResourceClient
+}
+
+func (m *mockListResourceStreamClient) Recv() (*proto.ListResource_Event, error) {
+	if m.current >= len(m.events) {
+		return nil, io.EOF
+	}
+
+	event := m.events[m.current]
+	m.current++
+	return event, nil
+}
+
+func TestGRPCProvider_ListResource(t *testing.T) {
+	client := mockProviderClient(t)
+	p := &GRPCProvider{
+		client: client,
+		ctx:    context.Background(),
+	}
+
+	// Create a mock stream client that will return resource events
+	mockStream := &mockListResourceStreamClient{
+		events: []*proto.ListResource_Event{
+			{
+				DisplayName: "Test Resource 1",
+				Identity: &proto.ResourceIdentityData{
+					IdentityData: &proto.DynamicValue{
+						Msgpack: []byte("\x81\xa7id_attr\xa4id-1"),
+					},
+				},
+			},
+			{
+				DisplayName: "Test Resource 2",
+				Identity: &proto.ResourceIdentityData{
+					IdentityData: &proto.DynamicValue{
+						Msgpack: []byte("\x81\xa7id_attr\xa4id-2"),
+					},
+				},
+				ResourceObject: &proto.DynamicValue{
+					Msgpack: []byte("\x81\xadresource_attr\xa5value"),
+				},
+			},
+		},
+	}
+
+	client.EXPECT().ListResource(
+		gomock.Any(),
+		gomock.Any(),
+	).Return(mockStream, nil)
+
+	// Create the request
+	configVal := cty.ObjectVal(map[string]cty.Value{
+		"config": cty.ObjectVal(map[string]cty.Value{
+			"filter_attr": cty.StringVal("filter-value"),
+		}),
+	})
+	request := providers.ListResourceRequest{
+		TypeName:              "list",
+		Config:                configVal,
+		IncludeResourceObject: true,
+		Limit:                 100,
+	}
+
+	resp := p.ListResource(request)
+	checkDiags(t, resp.Diagnostics)
+
+	data := resp.Result.AsValueMap()
+	if _, ok := data["data"]; !ok {
+		t.Fatal("Expected 'data' key in result")
+	}
+	// Verify that we received both events
+	if len(data["data"].AsValueSlice()) != 2 {
+		t.Fatalf("Expected 2 resources, got %d", len(data["data"].AsValueSlice()))
+	}
+	results := data["data"].AsValueSlice()
+
+	// Verify first event
+	displayName := results[0].GetAttr("display_name")
+	if displayName.AsString() != "Test Resource 1" {
+		t.Errorf("Expected DisplayName 'Test Resource 1', got '%s'", displayName.AsString())
+	}
+
+	expectedId1 := cty.ObjectVal(map[string]cty.Value{
+		"id_attr": cty.StringVal("id-1"),
+	})
+
+	identity := results[0].GetAttr("identity")
+	if !identity.RawEquals(expectedId1) {
+		t.Errorf("Expected Identity %#v, got %#v", expectedId1, identity)
+	}
+
+	// ResourceObject should be null for the first event as it wasn't provided
+	resourceObject := results[0].GetAttr("state")
+	if !resourceObject.IsNull() {
+		t.Errorf("Expected ResourceObject to be null, got %#v", resourceObject)
+	}
+
+	// Verify second event
+	displayName = results[1].GetAttr("display_name")
+	if displayName.AsString() != "Test Resource 2" {
+		t.Errorf("Expected DisplayName 'Test Resource 2', got '%s'", displayName.AsString())
+	}
+
+	expectedId2 := cty.ObjectVal(map[string]cty.Value{
+		"id_attr": cty.StringVal("id-2"),
+	})
+	identity = results[1].GetAttr("identity")
+	if !identity.RawEquals(expectedId2) {
+		t.Errorf("Expected Identity %#v, got %#v", expectedId2, identity)
+	}
+
+	expectedResource := cty.ObjectVal(map[string]cty.Value{
+		"resource_attr": cty.StringVal("value"),
+	})
+	resourceObject = results[1].GetAttr("state")
+	if !resourceObject.RawEquals(expectedResource) {
+		t.Errorf("Expected ResourceObject %#v, got %#v", expectedResource, resourceObject)
+	}
+}
+
+func TestGRPCProvider_ListResource_Error(t *testing.T) {
+	client := mockProviderClient(t)
+	p := &GRPCProvider{
+		client: client,
+		ctx:    context.Background(),
+	}
+
+	// Test case where the provider returns an error
+	client.EXPECT().ListResource(
+		gomock.Any(),
+		gomock.Any(),
+	).Return(nil, fmt.Errorf("provider error"))
+
+	configVal := cty.ObjectVal(map[string]cty.Value{
+		"config": cty.ObjectVal(map[string]cty.Value{
+			"filter_attr": cty.StringVal("filter-value"),
+		}),
+	})
+	request := providers.ListResourceRequest{
+		TypeName: "list",
+		Config:   configVal,
+	}
+
+	resp := p.ListResource(request)
+	checkDiagsHasError(t, resp.Diagnostics)
+}
+
+func TestGRPCProvider_ListResource_Diagnostics(t *testing.T) {
+	client := mockProviderClient(t)
+	p := &GRPCProvider{
+		client: client,
+		ctx:    context.Background(),
+	}
+
+	// Create a mock stream client that will return a resource event with diagnostics
+	mockStream := &mockListResourceStreamClient{
+		events: []*proto.ListResource_Event{
+			{
+				DisplayName: "Test Resource With Warning",
+				Identity: &proto.ResourceIdentityData{
+					IdentityData: &proto.DynamicValue{
+						Msgpack: []byte("\x81\xa7id_attr\xa4id-1"),
+					},
+				},
+				Diagnostic: []*proto.Diagnostic{
+					{
+						Severity: proto.Diagnostic_WARNING,
+						Summary:  "Test warning",
+						Detail:   "This is a test warning",
+					},
+				},
+			},
+		},
+	}
+
+	client.EXPECT().ListResource(
+		gomock.Any(),
+		gomock.Any(),
+	).Return(mockStream, nil)
+
+	// Create the request
+	configVal := cty.ObjectVal(map[string]cty.Value{
+		"config": cty.ObjectVal(map[string]cty.Value{
+			"filter_attr": cty.StringVal("filter-value"),
+		}),
+	})
+	request := providers.ListResourceRequest{
+		TypeName: "list",
+		Config:   configVal,
+		Limit:    100,
+	}
+
+	resp := p.ListResource(request)
+	checkDiags(t, resp.Diagnostics)
+
+	data := resp.Result.AsValueMap()
+	if _, ok := data["data"]; !ok {
+		t.Fatal("Expected 'data' key in result")
+	}
+
+	if !resp.Diagnostics.HasWarnings() {
+		t.Fatal("Expected warning diagnostics, but got none")
+	}
+}
+
+func TestGRPCProvider_ListResource_Limit(t *testing.T) {
+	client := mockProviderClient(t)
+	p := &GRPCProvider{
+		client: client,
+		ctx:    context.Background(),
+	}
+
+	// Create a mock stream client that will return resource events
+	mockStream := &mockListResourceStreamClient{
+		events: []*proto.ListResource_Event{
+			{
+				DisplayName: "Test Resource 1",
+				Identity: &proto.ResourceIdentityData{
+					IdentityData: &proto.DynamicValue{
+						Msgpack: []byte("\x81\xa7id_attr\xa4id-1"),
+					},
+				},
+			},
+			{
+				DisplayName: "Test Resource 2",
+				Identity: &proto.ResourceIdentityData{
+					IdentityData: &proto.DynamicValue{
+						Msgpack: []byte("\x81\xa7id_attr\xa4id-2"),
+					},
+				},
+			},
+			{
+				DisplayName: "Test Resource 3",
+				Identity: &proto.ResourceIdentityData{
+					IdentityData: &proto.DynamicValue{
+						Msgpack: []byte("\x81\xa7id_attr\xa4id-3"),
+					},
+				},
+			},
+		},
+	}
+
+	client.EXPECT().ListResource(
+		gomock.Any(),
+		gomock.Any(),
+	).Return(mockStream, nil)
+
+	// Create the request
+	configVal := cty.ObjectVal(map[string]cty.Value{
+		"config": cty.ObjectVal(map[string]cty.Value{
+			"filter_attr": cty.StringVal("filter-value"),
+		}),
+	})
+	request := providers.ListResourceRequest{
+		TypeName: "list",
+		Config:   configVal,
+		Limit:    2,
+	}
+
+	resp := p.ListResource(request)
+	checkDiags(t, resp.Diagnostics)
+
+	data := resp.Result.AsValueMap()
+	if _, ok := data["data"]; !ok {
+		t.Fatal("Expected 'data' key in result")
+	}
+	// Verify that we received both events
+	if len(data["data"].AsValueSlice()) != 2 {
+		t.Fatalf("Expected 2 resources, got %d", len(data["data"].AsValueSlice()))
+	}
+	results := data["data"].AsValueSlice()
+
+	// Verify that we received both events
+	if len(results) != 2 {
+		t.Fatalf("Expected 2 events, got %d", len(results))
+	}
+}
